Module niche

Module niche 

Logic related to the conversion of Option<T> to and from FFI-compatible representation

Structs

WithNiche
Marker for a type that has a niche value.

Enums

WithoutNiche
Marker for a type that has no trap representations and therefore no niche value

Traits

NicheStabilityKind