Module size

Module size 

Re-exports

pub use crate::Gt;
pub use crate::Zero;

Structs

MetaSized
Marker for types with a size that can be determined from pointer metadata.
Sized
Marker for types with a constant size known at compile time.

Enums

DynTraitLike
Marker for Trait objects or DSTs whose last field is a trait object.
ExternTypeLike
Marker for Extern types and DSTs whose last field is an extern type.
SliceLike
Marker for Slices or DSTs whose last field is a slice.

Traits

MetadataKind
SizedKind